@@ -4,7 +4,7 @@
 Code to analyse VASP defect calculation results, and other dope things.
 These functions are built from a combination of extremely useful modules
 from pymatgen and AIDE (by Adam Jackson and Alex Ganose), alongside
-substantial modification by myself, in the efforts of making an
+substantial modification, in the efforts of making an
 efficient, user-friendly package for managing and analysing defect
 calculations, with publication-quality outputs.
 """
@@ -23,6 +23,7 @@ from typing import Any
 import warnings
 import numpy as np
 import copy
+import pandas as pd
 
 import matplotlib.cm as cm
 import matplotlib.pyplot as plt
@@ -31,6 +32,7 @@ from matplotlib import rc
 
 from tabulate import tabulate
 from pymatgen.analysis.defects.thermodynamics import DefectPhaseDiagram
+from pymatgen.util.string import latexify, unicodeify
 from doped import aide_murphy_correction
 
 default_fonts = [
@@ -223,28 +225,39 @@ def formation_energy_table(
     show_key: bool = True,
     pd_facets: list = None,
 ):
+    """
+        Prints the formation energy tables for either a single chemical potential limit (i.e. phase
+        diagram facet) or each facet in the chempot_limits dict, depending on which version you
+        provide. Returns the results as either a pandas dataframe or list of dataframes.
+        """
     if "facets" in chempot_limits:
+        list_of_dfs = []
         if not pd_facets:
             pd_facets = chempot_limits["facets"].keys()  # Phase diagram facets to use for chemical
             # potentials, to tabulate formation energies
         for facet in pd_facets:
-            bold_print("Facet: " + facet)
-            single_formation_energy_table(
+            bold_print("Facet: " + unicodeify(facet))
+            df = single_formation_energy_table(
                 defect_phase_diagram,
                 chempots=chempot_limits["facets"][facet],
                 fermi_level=fermi_level,
                 hide_cols=hide_cols,
                 show_key=show_key,
             )
+            list_of_dfs.append(df)
             print("\n")
+
+        return list_of_dfs
+
     else:  # If you only want to give {Elt: Energy} dict for chempot_limits
-        single_formation_energy_table(
+        df = single_formation_energy_table(
             defect_phase_diagram,
             chempots=chempot_limits,
             fermi_level=fermi_level,
             hide_cols=hide_cols,
             show_key=show_key,
         )
+        return df
 
 
 def single_formation_energy_table(
@@ -254,12 +267,16 @@ def single_formation_energy_table(
     hide_cols: list = None,
     show_key: bool = True,
 ):
-    header = ["Defect", "Charge"]
+    """
+    Prints the formation energy table for a single chemical potential limit (i.e. phase diagram
+    facet), and returns the results as a pandas dataframe.
+    """
+    header = ["Defect", "Charge", "Defect Path"]
     table = []
     if hide_cols is None:
         hide_cols = []
     for defect_entry in defect_phase_diagram.entries:
-        row = [defect_entry.name, defect_entry.charge]
+        row = [defect_entry.name, defect_entry.charge, defect_entry.parameters["defect_path"]]
         if "Uncorrected_E" not in hide_cols:
             header += ["Uncorrected_E"]
             row += [f"{defect_entry.uncorrected_energy:.2f} eV"]
@@ -295,6 +312,13 @@ chempot_limits)(default: all 0) and the chosen fermi_level (default: 0)(i.e. at
         """
         )
 
+    sorted_df = pd.DataFrame(
+        table,
+        columns = ['Defect', 'Charge', 'defect_path', 'Uncorrected_E', 'Corrected_E', 'Formation_E']
+    )
+    sorted_df = sorted_df.sort_values('Formation_E')
+    return sorted_df
+
 
 def bold_print(string: str) -> None:
     """Does what it says on the tin. Prints the input string in bold."""
@@ -322,7 +346,7 @@ def formation_energy_plot(
     lg_fontsize=1.0,
     lg_position=None,
     fermi_level=None,
-    title="pd_facet_name",
+    title: str = None,
     saved=False,
     colormap="Dark2",
     minus_symbol="-",
@@ -340,15 +364,10 @@ def formation_energy_plot(
         for facet in pd_facets:
             mu_elts = chempot_limits["facets"][facet]
             elt_refs = chempot_limits["facets_wrt_elt_refs"][facet]
-            plot_filename = filename
-            if title == "pd_facet_name":
-                plot_title = facet
-            elif title:
-                plot_title = title
-                if not filename:
-                    plot_filename = plot_title + "_" + facet + ".pdf"
-            else:
-                plot_title = None
+            if not title:
+                title = facet
+            if not filename:
+                filename = title + "_" + facet + ".pdf"
 
             return _aide_pmg_plot(
                 defect_phase_diagram,
@@ -362,14 +381,14 @@ def formation_energy_plot(
                 lg_fontsize=lg_fontsize,
                 lg_position=lg_position,
                 fermi_level=fermi_level,
-                title=plot_title,
+                title=title,
                 saved=saved,
                 colormap=colormap,
                 minus_symbol=minus_symbol,
                 frameon=frameon,
                 chem_pot_table=chem_pot_table,
                 auto_labels=auto_labels,
-                filename=plot_filename,
+                filename=filename,
                 emphasis=emphasis,
             )
     else:  # If you only want to give {Elt: Energy} dict for chempot_limits, or no chempot_limits
@@ -718,14 +737,15 @@ some defects will have the same line colour). Recommended to change/set colormap
             )
 
     if title and chem_pot_table:
-        ax.set_title(title, size=1.2 * ax_fontsize * width, pad=28, fontdict={"fontweight": "bold"})
+        ax.set_title(latexify(title), size=1.2 * ax_fontsize * width, pad=28, fontdict={
+            "fontweight": "bold"})
     elif title:
-        ax.set_title("{}".format(title), size=ax_fontsize * width, fontdict={"fontweight": "bold"})
-    if saved or filename:
+        ax.set_title(latexify(title), size=ax_fontsize * width, fontdict={"fontweight": "bold"})
+    if saved and filename:
         if filename:
             plt.savefig(filename, bbox_inches="tight", dpi=600)
         else:
-            plt.savefig(str(title) + "_DWTB_plot.pdf", bbox_inches="tight", dpi=600)
+            plt.savefig(str(title) + "_doped_plot.pdf", bbox_inches="tight", dpi=600)
     return ax
 
 
@@ -873,6 +893,54 @@ def lany_zunger_corrected_defect_dict_from_freysoldt(defect_dict: dict):
     return lz_corrected_defect_dict
 
 
+def lany_zunger_corrected_defect_dict_from_kumagai(defect_dict: dict):
+    """Convert input parsed defect dictionary (presumably created using SingleDefectParser
+     from doped.pycdt.utils.parse_calculations) with Kumagai charge corrections to
+     the same parsed defect dictionary but with the 'Lany-Zunger' charge correction (same potential
+     alignment plus 0.65 * image charge correction.
+     Args:
+         parsed_defect_dict (dict):
+             Dictionary of parsed defect calculations (presumably created using SingleDefectParser
+             from doped.pycdt.utils.parse_calculations) (see example notebook)
+             Must have 'kumagai_meta' in defect.parameters for each charged defect (from
+             SingleDefectParser.kumagai_loader())
+     Returns:
+         Parsed defect dictionary with Lany-Zunger charge corrections.
+     """
+    random_defect_entry = list(defect_dict.values())[0]  # Just need any DefectEntry from
+    # defect_dict to get the lattice and dielectric matrix
+    lattice = random_defect_entry.bulk_structure.lattice.matrix
+    dielectric = random_defect_entry.parameters["dielectric"]
+    lz_image_charge_corrections = aide_murphy_correction.get_image_charge_correction(
+        lattice, dielectric
+    )
+    lz_corrected_defect_dict = copy.deepcopy(defect_dict)
+    for defect_name, defect_entry in lz_corrected_defect_dict.items():
+        if defect_entry.charge != 0:
+            potalign = defect_entry.parameters["kumagai_meta"][
+                "kumagai_potential_alignment_correction"
+            ]
+            makove_payne_pc_correction = lz_image_charge_corrections[abs(defect_entry.charge)]
+            defect_entry.parameters.update(
+                {
+                    "Lany-Zunger_Corrections": {
+                        "(Kumagai)_Potential_Alignment_Correction": potalign,
+                        "Makov-Payne_Image_Charge_Correction": makove_payne_pc_correction,
+                        "Lany-Zunger_Scaled_Image_Charge_Correction": 0.65
+                        * makove_payne_pc_correction,
+                        "Total_Lany-Zunger_Correction": potalign
+                        + 0.65 * makove_payne_pc_correction,
+                    }
+                }
+            )
+            defect_entry.corrections["charge_correction"] = defect_entry.parameters[
+                "Lany-Zunger_Corrections"
+            ]["Total_Lany-Zunger_Correction"]
+
+        lz_corrected_defect_dict.update({defect_name: defect_entry})
+    return lz_corrected_defect_dict
+
+
 def all_lines_formation_energy_plot(
     defect_phase_diagram,
     chempot_limits=None,
@@ -1165,13 +1233,15 @@ some defects will have the same line colour). Recommended to change/set colormap
             )
 
     if title and chem_pot_table:
-        ax.set_title(title, size=1.2 * ax_fontsize * width, pad=28, fontdict={"fontweight": "bold"})
+        ax.set_title(latexify(title), size=1.2 * ax_fontsize * width, pad=28, fontdict={
+            "fontweight":
+                                                                                   "bold"})
     elif title:
-        ax.set_title("{}".format(title), size=ax_fontsize * width, fontdict={"fontweight": "bold"})
+        ax.set_title(latexify(title), size=ax_fontsize * width, fontdict={"fontweight": "bold"})
     if saved or filename:
         if filename:
             plt.savefig(filename, bbox_inches="tight", dpi=600)
         else:
-            plt.savefig(str(title) + "_DWTB_plot.pdf", bbox_inches="tight", dpi=600)
+            plt.savefig(str(title) + "_doped_plot.pdf", bbox_inches="tight", dpi=600)
     else:
         return ax
