@@ -16,31 +16,11 @@ import logging
 import os
 import copy
 
-from pymatgen import Structure, Element
+from pymatgen.core.structure import Structure, Element
 from pymatgen.analysis.phase_diagram import PhaseDiagram
 from pymatgen.entries.computed_entries import ComputedStructureEntry
 from pymatgen.ext.matproj import MPRester
 
-from doped.pycdt.utils import parse_calculations
-
-#def get_vasprun(vasprun_path, **kwargs):
-#    """ Read the vasprun.xml(.gz) file as a pymatgen vasprun object """
-#    warnings.filterwarnings(
-#        "ignore", category=BadPotcarWarning
-#    )  # Ignore POTCAR warnings when loading vasprun.xml
-#    # pymatgen assumes the default PBE with no way of changing this within get_vasprun())
-#    warnings.filterwarnings("ignore", message="No POTCAR file with matching TITEL fields")
-#    if os.path.exists(vasprun_path):
-#        vasprun = Vasprun(vasprun_path)
-#    elif os.path.exists(vasprun_path + ".gz", **kwargs):
-#        vasprun = Vasprun(vasprun_path + ".gz", **kwargs)
-#    else:
-#        raise FileNotFoundError(
-#            f"""Well I can't fucking find a vasprun.xml(.gz) at {vasprun_path}(.gz).
-#                   You sure there's one there pal? I need it to parse the calculation results"""
-#        )
-#    return vasprun
-
 
 def get_mp_chempots_from_dpd(dpd):
     """
@@ -156,7 +136,7 @@ class MPChemPotAnalyzer(ChemPotAnalyzer):
         Args:
             bulk_ce: Pymatgen ComputedStructureEntry object for
                 bulk entry / supercell
-            subs_species (set): set of elemental species that are extrinsic to structure.
+            sub_species (set): set of elemental species that are extrinsic to structure.
                 Default is no subs included
             entries (dict): a dict of pymatgen ComputedEntry objects to build relevant phase diagram
                 The dict contains two keys: 'bulk_derived', and 'subs_set', each contains a list
@@ -356,7 +336,7 @@ class MPChemPotAnalyzer(ChemPotAnalyzer):
 
         return chem_lims
 
-    def get_chempots_from_composition(self, bulk_composition):
+    def get_chempots_from_composition(self, bulk_composition, full_sub_approach=False):
         """
         A simple method for getting GGA-PBE chemical potentials JUST
         from the composition information (Note: this only works if the
@@ -367,14 +347,33 @@ class MPChemPotAnalyzer(ChemPotAnalyzer):
                 object. This and mapi_key are only actual required input for
                 generating set of chemical potentials from Materials Project
                 database
+            full_sub_approach : Include sub_species in query, sub entries can
+                be found in self.entries['subs_set']
         """
         logger = logging.getLogger(__name__)
 
         redcomp = bulk_composition.reduced_composition
+        self.bulk_species_symbol = [s.symbol for s in redcomp.elements]
+
         if not self.entries:
-            self.bulk_species_symbol = [s.symbol for s in redcomp.elements]
-            with MPRester(api_key=self.mapi_key) as mp:
-                self.entries["bulk_derived"] = mp.get_entries_in_chemsys(self.bulk_species_symbol)
+            if full_sub_approach:  # this can be time consuming if several sub species exist
+                species_symbols = self.bulk_species_symbol[:]
+                for sub_el in self.sub_species:
+                    species_symbols.append(sub_el)
+
+                with MPRester(api_key=self.mapi_key) as mp:
+                    self.entries["bulk_derived"] = mp.get_entries_in_chemsys(species_symbols)
+
+                self.entries["subs_set"] = {sub_el: [] for sub_el in self.sub_species}
+                for entry in self.entries["bulk_derived"]:
+                    for sub_el in self.sub_species:
+                        if sub_el in entry.composition:
+                            self.entries["subs_set"][sub_el].append(entry)
+
+            else:
+                with MPRester(api_key=self.mapi_key) as mp:
+                    self.entries["bulk_derived"] = mp.get_entries_in_chemsys(
+                        self.bulk_species_symbol)
 
         pd = PhaseDiagram(self.entries["bulk_derived"])
         chem_lims = pd.get_all_chempots(redcomp)
@@ -526,7 +525,8 @@ class UserChemPotAnalyzer(ChemPotAnalyzer):
                     os.path.join(pdfile, structfile, "vasprun.xml.gz")):
                 try:
                     print("loading ", structfile)
-                    vr = parse_calculations.get_vasprun(os.path.join(pdfile, structfile,
+                    from doped.pycdt.utils.parse_calculations import get_vasprun
+                    vr = get_vasprun(os.path.join(pdfile, structfile,
                                                                      "vasprun.xml"))
                     entry_from_vr = vr.get_computed_entry()
                     entry_from_vr.data.update({"Orig_Folder_Name": structfile})
@@ -539,7 +539,8 @@ class UserChemPotAnalyzer(ChemPotAnalyzer):
             vr_path = os.path.join(self.path_base, "bulk", "vasprun.xml")
             if os.path.exists(vr_path):
                 print("loading bulk computed entry")
-                bulkvr = parse_calculations.get_vasprun(vr_path)
+                from doped.pycdt.utils.parse_calculations import get_vasprun
+                bulkvr = get_vasprun(vr_path)
                 self.bulk_ce = bulkvr.get_computed_entry()
             else:
                 print(
@@ -762,7 +763,7 @@ class UserChemPotInputGenerator(object):
         self.sub_species = sub_species
         self.path_base = path_base
         self.mapi_key = mapi_key
-        self.MPC = MPChemPotAnalyzer()
+        self.MPC = MPChemPotAnalyzer(sub_species=sub_species)
 
     def setup_phase_diagram_calculations(
         self,
@@ -772,6 +773,7 @@ class UserChemPotInputGenerator(object):
         write_files=False,
         overwrite=False,
         include_elements=True,
+        full_sub_approach=True
     ):
         """
         This method allows for setting up local phase diagram calculations so a user can calculate
@@ -803,26 +805,28 @@ class UserChemPotInputGenerator(object):
         overwrite: write files even if PhaseDiagram folder already exists. Defaults to False.
 
         include_elements: set up all elemental reference phases as well (necessary to properly
-        determine absolute
-                          chemical potentials, zero-reference elemental energies etc.),
-                          regardless of whether the
-                          elemental phase is a local facet adjacent to the MP GGA-calculated
-                          stable phase space for the
-                          composition of interest. Defaults to True.
+            determine absolute chemical potentials, zero-reference elemental energies etc.),
+            regardless of whether the elemental phase is a local facet adjacent to the MP
+            GGA-calculated stable phase space for the composition of interest. Defaults to True.
+
+        full_sub_approach : Include sub_species in query, sub entries can then be found in
+            self.MPC.entries['subs_set']
 
         """
 
         # while GGA chem pots won't be used here; use this method for quickly gathering phase
         # diagram object entries AND to find phases of interest if you just want to re-calculate
         # local facets
-        MPgga_muvals = self.MPC.get_chempots_from_composition(self.bulk_composition)
+        if full_sub_approach and self.sub_species:
+            MPgga_muvals = self.MPC.get_chempots_from_composition(self.bulk_composition,
+                                                                  full_sub_approach=True)
+        else:
+            MPgga_muvals = self.MPC.get_chempots_from_composition(self.bulk_composition)
 
         if full_phase_diagram:
             setupphases = set(
                 [
-                    localentry.name
-                    for entrykey in self.MPC.entries.keys()
-                    for localentry in self.MPC.entries[entrykey]
+                    localentry.name for localentry in self.MPC.entries['bulk_derived']
                 ]
             )  # all elements in
             # phase diagram
@@ -842,8 +846,7 @@ class UserChemPotInputGenerator(object):
                 )  # just local facets
 
             if include_elements:  # add elemental reference phases to structures to setup
-                for entrykey in self.MPC.entries.keys():
-                    for localentry in self.MPC.entries[entrykey]:
+                for localentry in self.MPC.entries['bulk_derived']:
                         if localentry.is_element:
                             setupphases.add(localentry.name)  # set.add() only adds entry if not
                             # already in set
@@ -854,7 +857,7 @@ class UserChemPotInputGenerator(object):
         # create phase diagram object for analyzing PBE-GGA energetics of structures computed in
         # MP database
         full_structure_entries = [
-            struct for entrykey in self.MPC.entries.keys() for struct in self.MPC.entries[entrykey]
+            struct for struct in self.MPC.entries['bulk_derived']
         ]
         pd = PhaseDiagram(full_structure_entries)
 
@@ -889,7 +892,7 @@ class UserChemPotInputGenerator(object):
 
         # Set up structure files locally if desired
         if not write_files:
-            print("Returning chempot structures, but no making POSCAR files.")
+            print("Returning chempot structures, but not making POSCAR files.")
         else:
             if os.path.exists(os.path.join(self.path_base, "PhaseDiagram")) and not overwrite:
                 print("PhaseDiagram folder already exists! Set overwrite = True if you want me to overwrite it")
