@@ -29,7 +29,7 @@ from pymatgen.ext.matproj import MPRester
 from pymatgen.io.vasp.inputs import Potcar, UnknownPotcarWarning
 from pymatgen.io.vasp.outputs import Vasprun, Locpot, Outcar, Poscar
 
-from doped.pycdt.core.chemical_potentials import MPChemPotAnalyzer
+from doped.pycdt.core import chemical_potentials
 
 
 def convert_cd_to_de(cd, b_cse):
@@ -62,7 +62,7 @@ def convert_cd_to_de(cd, b_cse):
         defect_site = PeriodicSite.from_dict(site_cls)
 
     poss_deflist = sorted(
-        bulk_sc_structure.get_sites_in_sphere(defect_site.coords, 0.1, include_index=True),
+        bulk_sc_structure.get_sites_in_sphere(defect_site.coords, 0.2, include_index=True),
         key=lambda x: x[1],
     )
     if len(poss_deflist) != 1:
@@ -167,8 +167,8 @@ class SingleDefectParser:
         Identify defect object based on file paths. Minimal parsing performing for
         instantiating the SingleDefectParser class.
 
-        :param path_to_defect (str): path to defect file of interest (with vasprun.xml(.gz))
-        :param path_to_bulk (str): path to bulk file of interest (with vasprun.xml(.gz))
+        :param path_to_defect (str): path to defect folder of interest (with vasprun.xml(.gz))
+        :param path_to_bulk (str): path to bulk folder of interest (with vasprun.xml(.gz))
         :param dielectric (float or 3x3 matrix): ionic + static contributions to dielectric constant
         :param defect_charge (int):
         :param mpid (str):
@@ -200,6 +200,9 @@ class SingleDefectParser:
         else:
             initial_defect_structure = defect_vr.initial_structure.copy()
 
+        # Add initial defect structure to parameters, so it can be pulled later on (eg. for Kumagai loader)
+        parameters["initial_defect_structure"] = initial_defect_structure
+
         # identify defect site, structural information, and create defect object
         num_ids = len(initial_defect_structure)
         num_bulk = len(bulk_sc_structure)
@@ -212,15 +215,24 @@ class SingleDefectParser:
         else:
             raise ValueError(
                 "Could not identify defect type just from number of sites in structure: "
-                "{} in bulk vs. {} in defect?".format(num_ids, num_bulk)
+                "{} in bulk vs. {} in defect?".format(num_bulk, num_ids)
             )
 
         defect_index_sc_coords = None
         transformation_path = os.path.join(path_to_defect, "transformation.json")
+        if not os.path.exists(transformation_path): # try next folder up
+            orig_transformation_path = transformation_path
+            transformation_path = os.path.join(
+                os.path.dirname(os.path.normpath(path_to_defect)), "transformation.json"
+            )
+            if os.path.exists(transformation_path):
+                print("No transformation file found at {}, but found one at {}. "
+                      "Using this for defect parsing.".format(orig_transformation_path, transformation_path))
+
         if os.path.exists(transformation_path):
             tf = loadfn(transformation_path)
             site = tf["defect_supercell_site"]
-            if defect_type == "Vacancy":
+            if defect_type == "Vacancy": # should we also search the bulk structure for substitutions?
                 poss_deflist = sorted(
                     bulk_sc_structure.get_sites_in_sphere(site.coords, 0.2, include_index=True),
                     key=lambda x: x[1],
@@ -242,8 +254,8 @@ class SingleDefectParser:
             defect_index_sc_coords = poss_deflist[0][2]
         else:
             print(
-                "No transformation file exists at {}.\nCalculating defect index manually"
-                " (proceed with caution)".format(transformation_path)
+                "No transformation file exists at {} or {}.\nCalculating defect index manually"
+                " (proceed with caution)".format(orig_transformation_path, transformation_path)
             )
 
         # IF not transformation file exists, the defect_index_sc_coords will not be identified in
@@ -265,7 +277,7 @@ class SingleDefectParser:
             poss_defect = []
             if defect_type in ["Vacancy", "Interstitial"]:
                 for mindist, bulk_index, defect_index in min_dist_with_index:
-                    if mindist < 0.1:
+                    if mindist < 0.2:
                         site_matching_indices.append([bulk_index, defect_index])
                     elif defect_type == "Vacancy":
                         poss_defect.append([bulk_index, bulksites[bulk_index][:]])
@@ -284,7 +296,7 @@ class SingleDefectParser:
                         bulk_sc_structure[bulk_index].specie
                         == initial_defect_structure[defect_index].specie
                     )
-                    if mindist < 0.1 and species_match:
+                    if mindist < 0.2 and species_match:
                         site_matching_indices.append([bulk_index, defect_index])
 
                     elif not species_match:
@@ -410,6 +422,11 @@ class SingleDefectParser:
         bulk_atomic_site_averages = bulk_outcar.electrostatic_potential
         defect_atomic_site_averages = def_outcar.electrostatic_potential
 
+        # Is parsing the bulk structure and defect site here again unnecessary?
+        # Yes, if the user has used SingleDefectParser.from_paths, but not if they've
+        # been a cowboy and used SingleDefectParser() with a pre-generated DefectEntry...
+        # So let's keep it for the moment
+
         if os.path.exists(os.path.join(self.defect_entry.parameters["bulk_path"], "POSCAR")):
             bulk_sc_structure = Poscar.from_file(
                 os.path.join(self.defect_entry.parameters["bulk_path"], "POSCAR")
@@ -421,17 +438,6 @@ class SingleDefectParser:
                 os.path.join(self.defect_entry.parameters["bulk_path"], "vasprun.xml")
             ).initial_structure.copy()
 
-        if self.defect_entry.site:
-            defect_frac_sc_coords = self.defect_entry.site.frac_coords
-            poss_deflist = sorted(
-                bulk_sc_structure.get_sites_in_sphere(
-                    self.defect_entry.site.coords, 0.2, include_index=True
-                ),
-                key=lambda x: x[1],
-            )
-            defect_index_sc_coords = poss_deflist[0][2]
-        # May add this bit if it's still suggesting several possible defect sites
-
         if "initial_defect_structure" in self.defect_entry.parameters:
             initial_defect_structure = self.defect_entry.parameters["initial_defect_structure"]
         elif os.path.exists(os.path.join(self.defect_entry.parameters["defect_path"], "POSCAR")):
@@ -457,6 +463,7 @@ class SingleDefectParser:
 
         site_matching_indices = []
         poss_defect = []
+        defect_index_sc_coords = None
         if isinstance(self.defect_entry.defect, (Vacancy, Interstitial)):
             for mindist, bulk_index, defect_index in min_dist_with_index:
                 if mindist < 0.2:
@@ -483,7 +490,7 @@ class SingleDefectParser:
                     bulk_sc_structure[bulk_index].specie
                     == initial_defect_structure[defect_index].specie
                 )
-                if mindist < 0.1 and species_match:
+                if mindist < 0.2 and species_match:
                     site_matching_indices.append([bulk_index, defect_index])
 
                 elif not species_match and defect_index_sc_coords is None:
@@ -493,6 +500,32 @@ class SingleDefectParser:
             if len(poss_defect) == 1:
                 defect_index_sc_coords = poss_defect[0][0]
                 defect_frac_sc_coords = poss_defect[0][1]
+
+            elif self.defect_entry.site:
+                defect_frac_sc_coords = self.defect_entry.site.frac_coords
+                if defect_type == "Vacancy":
+                    poss_deflist = sorted(
+                        bulk_sc_structure.get_sites_in_sphere(
+                            self.defect_entry.site.coords, 0.2, include_index=True
+                        ),
+                        key=lambda x: x[1],
+                    )
+                    searched = "bulk_supercell"
+                else:
+                    poss_deflist = sorted(
+                        initial_defect_structure.get_sites_in_sphere(
+                            self.defect_entry.site.coords, 0.2, include_index=True
+                        ),
+                        key=lambda x: x[1],
+                    )
+                    searched = "initial_defect_structure"
+                if not poss_deflist:
+                    raise ValueError(
+                        "Could not find defect site {} in {}, abandoning parsing".format(
+                            self.defect_entry.site, searched)
+                    )
+                defect_index_sc_coords = poss_deflist[0][2]
+
             else:
                 raise ValueError(
                     "Found {} possible defect sites when matching bulk and "
@@ -653,8 +686,6 @@ class SingleDefectParser:
         vbm, cbm, bandgap = None, None, None
         gap_parameters = {}
         if mpid is not None and not no_MP:
-            # TODO: NEED to be smarter about use of +U or HSE etc in MP gga band structure
-            #  calculations...
             with MPRester() as mp:
                 bs = mp.get_bandstructure_by_material_id(mpid)
             if bs:
@@ -688,7 +719,8 @@ class SingleDefectParser:
                     "bulk calculation."
                 )
                 print(
-                    "Note that it would be better to " "perform real band structure calculation..."
+                    "This may not be appropriate if the VBM/CBM occur at reciprocal points "
+                    "not included in the bulk calculation."
                 )
 
             gap_parameters.update({"MP_gga_BScalc_data": None})  # to signal no MP BS is used
@@ -713,6 +745,9 @@ class SingleDefectParser:
         self.defect_entry.parameters.update(gap_parameters)
 
     def run_compatibility(self):
+        # Set potalign so pymatgen can calculate bandfilling for 'neutral' defects (possible for resonant dopants etc.)
+        if self.defect_entry.charge == 0 and "potalign" not in self.defect_entry.parameters:
+            self.defect_entry.parameters["potalign"] = 0
         self.defect_entry = self.compatibility.process_entry(self.defect_entry)
         if not self.defect_entry.parameters["is_compatible"]:
             delocalized_warning = f"""
@@ -720,7 +755,7 @@ Delocalization analysis has indicated that {self.defect_entry.name}
 with charge {self.defect_entry.charge} may not be compatible with the chosen charge correction
 scheme, and may require a larger supercell for accurate calculation of the energy. Recommended to
 look at the correction plots (i.e. run `get_correction_freysoldt(DefectEntry,...,plot=True)` from
-`DefectsWithTheBoys.pycdt.corrections.finite_size_charge_correction`) to visually determine if
+`doped.pycdt.corrections.finite_size_charge_correction`) to visually determine if
 charge correction scheme still appropriate, then `sdp.compatibility.perform_freysoldt(DefectEntry)`
 if you're happy (replace 'freysoldt' with 'kumagai' if using anisotropic correction).
 You can also change the DefectCompatibility() tolerance settings via the `compatibility` parameter
@@ -948,7 +983,8 @@ class PostProcess:
                     "No mp-id provided, will fetch CBM/VBM details from the " "bulk calculation."
                 )
             logger.warning(
-                "Note that it would be better to " "perform real band structure calculation..."
+                "This may not be appropriate if the VBM/CBM occur at reciprocal points "
+                "not included in the bulk calculation."
             )
             vr = get_vasprun(
                 os.path.join(self._root_fldr, "bulk", "vasprun.xml"), parse_potcar_file=False
@@ -971,7 +1007,7 @@ class PostProcess:
         logger = logging.getLogger(__name__)
 
         if self._mpid:
-            cpa = MPChemPotAnalyzer(
+            cpa = chemical_potentials.MPChemPotAnalyzer(
                 mpid=self._mpid, sub_species=self._substitution_species, mapi_key=self._mapi_key
             )
         else:
@@ -982,7 +1018,7 @@ class PostProcess:
                 msg = "Could not fetch computed entry for atomic chempots!"
                 logger.warning(msg)
                 raise ValueError(msg)
-            cpa = MPChemPotAnalyzer(
+            cpa = chemical_potentials.MPChemPotAnalyzer(
                 bulk_ce=bulkvr.get_computed_entry(),
                 sub_species=self._substitution_species,
                 mapi_key=self._mapi_key,
