@@ -6,28 +6,38 @@ Code to generate VASP defect calculation input files.
 
 __author__ = "Seán Kavanagh"
 __copyright__ = "MIT License"
-__version__ = "0.0.1"
+__version__ = "0.0.3"
 __maintainer__ = "Seán Kavanagh"
 __email__ = "sean.kavanagh.19@ucl.ac.uk"
 __date__ = "May 19, 2020"
 
 import functools
 import os
+from copy import deepcopy # See https://stackoverflow.com/a/22341377/14020960 why
 import warnings
 from typing import TYPE_CHECKING
+import numpy as np
 
 from monty.io import zopen
-from monty.serialization import dumpfn
+from monty.serialization import dumpfn, loadfn
 from pymatgen.io.vasp import Incar, Kpoints, Poscar
+from pymatgen.io.vasp.inputs import incar_params, BadIncarWarning, Kpoints_supported_modes
 from pymatgen.io.vasp.sets import DictSet, BadInputSetWarning
+from ase.dft.kpoints import monkhorst_pack
 
-from doped.pycdt.utils.vasp import DefectRelaxSet
+from doped.pycdt.utils.vasp import DefectRelaxSet, _check_psp_dir
 
 
 if TYPE_CHECKING:
     import pymatgen.core.periodic_table
     import pymatgen.core.structure
 
+MODULE_DIR = os.path.dirname(os.path.abspath(__file__))
+default_potcar_dict = loadfn(os.path.join(MODULE_DIR, "default_POTCARs.yaml"))
+
+def scaled_ediff(natoms): # 1e-5 for 50 atoms, up to max 1e-4
+    ediff = float(f"{((natoms/50)*1e-5):.1g}")
+    return ediff if ediff <= 1e-4 else 1e-4
 
 def prepare_vasp_defect_inputs(defects: dict) -> dict:
     """
@@ -59,18 +69,6 @@ def prepare_vasp_defect_inputs(defects: dict) -> dict:
 
             defect_relax_set = DefectRelaxSet(supercell["structure"], charge=charge)
 
-            # try: # Actually, we don't need this code block
-            #     potcar = defect_relax_set.potcar
-            # except AttributeError as error:
-            #     print(
-            #         f"""Error {error} occurred with arguments {error.args},
-            #     meaning no POTCAR generated, have you set the PseudoPotential directory
-            #     in your .pmgrc.yaml file? (See https://bitbucket.org/mbkumar/pycdt).
-            #     """
-            #     )
-            #     break
-
-            incar = defect_relax_set.incar
             poscar = defect_relax_set.poscar
             struct = defect_relax_set.structure
             poscar.comment = (
@@ -81,16 +79,11 @@ def prepare_vasp_defect_inputs(defects: dict) -> dict:
             )
             folder_name = defect["name"] + f"_{charge}"
             print(folder_name)
-            try:
-                # Only set if change in NELECT
-                nelect = incar.as_dict()["NELECT"]
-            except KeyError:
-                # Get NELECT if no change (KV = -dNELECT = 0)
-                nelect = defect_relax_set.nelect
+
             defect_input_dict[folder_name] = {
                 "Defect Structure": struct,
-                "NELECT": nelect,
                 "POSCAR Comment": poscar.comment,
+                "Transformation Dict": dict_transf
             }
     return defect_input_dict
 
@@ -155,7 +148,8 @@ def prepare_vasp_defect_dict(
 
 # noinspection DuplicatedCode
 def vasp_gam_files(
-    single_defect_dict: dict, input_dir: str = None, incar_settings: dict = None
+    single_defect_dict: dict, input_dir: str = None, incar_settings: dict = None,
+        potcar_settings: dict = None
 ) -> None:
     """
     Generates input files for VASP Gamma-point-only rough relaxation
@@ -174,13 +168,54 @@ def vasp_gam_files(
             Highly recommended to look at output INCARs or doped.vasp_input
             source code, to see what the default INCAR settings are.
             (default: None)
+        potcar_settings (dict):
+            Dictionary of user POTCAR settings to override default settings.
+            Highly recommended to look at `default_potcar_dict` from doped.vasp_input to see what
+            the (Pymatgen) syntax and doped default settings are.
+            (default: None)
     """
     supercell = single_defect_dict["Defect Structure"]
-    nelect = single_defect_dict["NELECT"]
     poscar_comment = (
-        single_defect_dict["POSCAR Comment"] if single_defect_dict["POSCAR Comment"] else None
+        single_defect_dict["POSCAR Comment"] if "POSCAR Comment" in single_defect_dict else None
     )
 
+    # Directory
+    vaspgaminputdir = input_dir + "/vasp_gam/" if input_dir else "VASP_Files/vasp_gam/"
+    if not os.path.exists(vaspgaminputdir):
+        os.makedirs(vaspgaminputdir)
+
+    warnings.filterwarnings(
+        "ignore", category=BadInputSetWarning
+    )  # Ignore POTCAR warnings because Pymatgen incorrectly detecting POTCAR types
+    potcar_dict = deepcopy(default_potcar_dict)
+    if potcar_settings:
+        potcar_dict["POTCAR"].update(potcar_settings.pop("POTCAR"))
+        potcar_dict.update(potcar_settings)
+    defect_relax_set = DefectRelaxSet(supercell, charge=single_defect_dict["Transformation "
+                                                                           "Dict"]["charge"],
+                                      user_potcar_settings=potcar_dict["POTCAR"],
+                                      user_potcar_functional=potcar_dict["POTCAR_FUNCTIONAL"])
+    potcars = _check_psp_dir()
+    if potcars:
+        defect_relax_set.potcar.write_file(vaspgaminputdir + "POTCAR")
+    else: # make the folders without POTCARs
+        warnings.warn("POTCAR directory not set up with pymatgen, so only POSCAR files will be "
+                      "generated (POTCARs also needed to determine appropriate NELECT setting in "
+                      "INCAR files)")
+        vaspgamposcar = defect_relax_set.poscar
+        if poscar_comment:
+            vaspgamposcar.comment = poscar_comment
+        vaspgamposcar.write_file(vaspgaminputdir + "POSCAR")
+        return  # exit here
+
+    relax_set_incar = defect_relax_set.incar
+    try:
+        # Only set if change in NELECT
+        nelect = relax_set_incar.as_dict()["NELECT"]
+    except KeyError:
+        # Get NELECT if no change (KV = -dNELECT = 0)
+        nelect = defect_relax_set.nelect
+
     # Variable parameters first
     vaspgamincardict = {
         "# May need to change NELECT, IBRION, NCORE, KPAR, AEXX, ENCUT, NUPDOWN, ISPIN, "
@@ -196,11 +231,11 @@ def vasp_gam_files(
         "POTIM": 0.2,
         "ISPIN": 2,
         "ICORELEVEL": "0 # Needed if using the Kumagai-Oba (eFNV) anisotropic charge correction",
-        "# LSUBROT": "True # Uncomment this if relaxation doesn't converge",
+        "LSUBROT": "False # Change to True if relaxation poorly convergent",
         "ALGO": "All",
         "ADDGRID": True,
-        "EDIFF": 5e-06,
-        "EDIFFG": "-0.005 # Tight relax criteria for vasp_gam, it's cheap enough buddy",
+        "EDIFF": f"{scaled_ediff(supercell.num_sites)} # May need to reduce for tricky relaxations",
+        "EDIFFG": -0.01,
         "HFSCREEN": 0.2,
         "ICHARG": 1,
         "ISIF": 2,
@@ -220,118 +255,20 @@ def vasp_gam_files(
         "SIGMA": 0.05,
     }
     if incar_settings:
+        for k in incar_settings.keys():  # check INCAR flags and warn if they don't exist (typos)
+            if k not in incar_params.keys():  # this code is taken from pymatgen.io.vasp.inputs
+                warnings.warn(  # but only checking keys, not values so we can add comments etc
+                    "Cannot find %s from your incar_settings in the list of INCAR flags" % (k),
+                    BadIncarWarning,
+                )
         vaspgamincardict.update(incar_settings)
 
-    # Directory
-    vaspgaminputdir = input_dir + "/vasp_gam/" if input_dir else "VASP_Files/vasp_gam/"
-    if not os.path.exists(vaspgaminputdir):
-        os.makedirs(vaspgaminputdir)
-
-    vasppotcardict = {
-        "POTCAR_FUNCTIONAL": "PBE",  # May need to change this if you've got
-        # different POTCARs
-        "POTCAR": {
-            "Ac": "Ac",
-            "Ag": "Ag",
-            "Al": "Al",
-            "Ar": "Ar",
-            "As": "As",
-            "Au": "Au",
-            "B": "B",
-            "Ba": "Ba_sv",
-            "Be": "Be_sv",
-            "Bi": "Bi",
-            "Br": "Br",
-            "C": "C",
-            "Ca": "Ca_sv",
-            "Cd": "Cd",
-            "Ce": "Ce",
-            "Cl": "Cl",
-            "Co": "Co",
-            "Cr": "Cr_pv",
-            "Cs": "Cs_sv",
-            "Cu": "Cu_pv",
-            "Dy": "Dy_3",
-            "Er": "Er_3",
-            "Eu": "Eu",
-            "F": "F",
-            "Fe": "Fe_pv",
-            "Ga": "Ga_d",
-            "Gd": "Gd",
-            "Ge": "Ge_d",
-            "H": "H",
-            "He": "He",
-            "Hf": "Hf_pv",
-            "Hg": "Hg",
-            "Ho": "Ho_3",
-            "I": "I",
-            "In": "In_d",
-            "Ir": "Ir",
-            "K": "K_sv",
-            "Kr": "Kr",
-            "La": "La",
-            "Li": "Li_sv",
-            "Lu": "Lu_3",
-            "Mg": "Mg_pv",
-            "Mn": "Mn_pv",
-            "Mo": "Mo_pv",
-            "N": "N",
-            "Na": "Na_pv",
-            "Nb": "Nb_pv",
-            "Nd": "Nd_3",
-            "Ne": "Ne",
-            "Ni": "Ni_pv",
-            "Np": "Np",
-            "O": "O",
-            "Os": "Os_pv",
-            "P": "P",
-            "Pa": "Pa",
-            "Pb": "Pb_d",
-            "Pd": "Pd",
-            "Pm": "Pm_3",
-            "Pr": "Pr_3",
-            "Pt": "Pt",
-            "Pu": "Pu",
-            "Rb": "Rb_sv",
-            "Re": "Re_pv",
-            "Rh": "Rh_pv",
-            "Ru": "Ru_pv",
-            "S": "S",
-            "Sb": "Sb",
-            "Sc": "Sc_sv",
-            "Se": "Se",
-            "Si": "Si",
-            "Sm": "Sm_3",
-            "Sn": "Sn_d",
-            "Sr": "Sr_sv",
-            "Ta": "Ta_pv",
-            "Tb": "Tb_3",
-            "Tc": "Tc_pv",
-            "Te": "Te",
-            "Th": "Th",
-            "Ti": "Ti_pv",
-            "Tl": "Tl_d",
-            "Tm": "Tm_3",
-            "U": "U",
-            "V": "V_pv",
-            "W": "W_pv",
-            "Xe": "Xe",
-            "Y": "Y_sv",
-            "Yb": "Yb_2",
-            "Zn": "Zn",
-            "Zr": "Zr_sv",
-        },
-    }
     vaspgamkpts = Kpoints().from_dict(
         {"comment": "Kpoints from doped.vasp_gam_files", "generation_style": "Gamma"}
     )
     vaspgamincar = Incar.from_dict(vaspgamincardict)
-    warnings.filterwarnings(
-        "ignore", category=BadInputSetWarning
-    )  # Ignore POTCAR warnings because Pymatgen incorrectly detecting POTCAR types
-    vaspgaminput = DictSet(supercell, config_dict=vasppotcardict)
-    vaspgamposcar = vaspgaminput.poscar
-    vaspgaminput.potcar.write_file(vaspgaminputdir + "POTCAR")
+
+    vaspgamposcar = defect_relax_set.poscar
     if poscar_comment:
         vaspgamposcar.comment = poscar_comment
     vaspgamposcar.write_file(vaspgaminputdir + "POSCAR")
@@ -345,11 +282,12 @@ def vasp_std_files(
     input_dir: str = None,
     incar_settings: dict = None,
     kpoints_settings: dict = None,
+    potcar_settings: dict = None,
 ) -> None:
     """
-    Generates INCAR and KPOINTS for vasp_std expensive k-point mesh relaxation.
-    For POSCAR and POTCAR, use on command-line (to continue on from vasp_gam run):
-    'cp vasp_gam/CONTCAR vasp_std/POSCAR; cp vasp_gam/{POTCAR,CHGCAR} vasp_std/'
+    Generates INCAR, POTCAR and KPOINTS for vasp_std expensive k-point mesh relaxation.
+    For POSCAR, use on command-line (to continue on from vasp_gam run):
+    'cp vasp_gam/CONTCAR vasp_std/POSCAR; cp vasp_gam/CHGCAR vasp_std/'
     Args:
         single_defect_dict (dict):
             Single defect-dictionary from prepare_vasp_defect_inputs()
@@ -370,8 +308,47 @@ def vasp_std_files(
             and/or "kpoints": [[3, 3, 1]] etc.
             Default KPOINTS is Gamma-centred 2 x 2 x 2 mesh.
             (default: None)
+        potcar_settings (dict):
+            Dictionary of user POTCAR settings to override default settings.
+            Highly recommended to look at `default_potcar_dict` from doped.vasp_input to see what
+            the (Pymatgen) syntax and doped default settings are.
+            (default: None)
     """
-    nelect = single_defect_dict["NELECT"]
+    supercell = single_defect_dict["Defect Structure"]
+
+    # Directory
+    vaspstdinputdir = input_dir + "/vasp_std/" if input_dir else "VASP_Files/vasp_std/"
+    if not os.path.exists(vaspstdinputdir):
+        os.makedirs(vaspstdinputdir)
+
+    warnings.filterwarnings(
+        "ignore", category=BadInputSetWarning
+    )  # Ignore POTCAR warnings because Pymatgen incorrectly detecting POTCAR types
+
+    potcar_dict = deepcopy(default_potcar_dict)
+    if potcar_settings:
+        potcar_dict["POTCAR"].update(potcar_settings.pop("POTCAR"))
+        potcar_dict.update(potcar_settings)
+    defect_relax_set = DefectRelaxSet(supercell, charge=single_defect_dict["Transformation "
+                                                                           "Dict"]["charge"],
+                                      user_potcar_settings=potcar_dict["POTCAR"],
+                                      user_potcar_functional=potcar_dict["POTCAR_FUNCTIONAL"])
+    potcars = _check_psp_dir()
+    if not potcars:
+        warnings.warn("POTCAR directory not set up with pymatgen, so no input files will be "
+                      "generated (you should use vasp_input.vasp_gam_files to create the initial "
+                      "relaxation files, then continue from this pre-converged structure with "
+                      "vasp_std)")
+        return  # exit here
+    defect_relax_set.potcar.write_file(vaspstdinputdir + "POTCAR")
+
+    relax_set_incar = defect_relax_set.incar
+    try:
+        # Only set if change in NELECT
+        nelect = relax_set_incar.as_dict()["NELECT"]
+    except KeyError:
+        # Get NELECT if no change (KV = -dNELECT = 0)
+        nelect = defect_relax_set.nelect
 
     # Variable parameters first
     vaspstdincardict = {
@@ -387,11 +364,11 @@ def vasp_std_files(
         "POTIM": 0.2,
         "ISPIN": "2 # Check mag in OSZICAR though, if 0 don't bother with spin polarisation ("
         "change to 1",
-        "LSUBROT": True,
+        "LSUBROT": "False # Change to True if relaxation poorly convergent",
         "ICORELEVEL": "0 # Get core potentials in OUTCAR for Kumagai corrections",
         "ALGO": "All",
         "ADDGRID": True,
-        "EDIFF": "1e-05 # May need to reduce for tricky relaxations",
+        "EDIFF": f"{scaled_ediff(supercell.num_sites)} # May need to reduce for tricky relaxations",
         "EDIFFG": -0.01,
         "HFSCREEN": 0.2,
         "IBRION": "1 # May need to change to 2 for difficult relaxations though",
@@ -413,13 +390,14 @@ def vasp_std_files(
         "SIGMA": 0.05,
     }
     if incar_settings:
+        for k in incar_settings.keys():  # check INCAR flags and warn if they don't exist (typos)
+            if k not in incar_params.keys():  # this code is taken from pymatgen.io.vasp.inputs
+                warnings.warn(  # but only checking keys, not values so we can add comments etc
+                    "Cannot find %s from your incar_settings in the list of INCAR flags" % (k),
+                    BadIncarWarning,
+                )
         vaspstdincardict.update(incar_settings)
 
-    # Directory
-    vaspstdinputdir = input_dir + "/vasp_std/" if input_dir else "VASP_Files/vasp_std/"
-    if not os.path.exists(vaspstdinputdir):
-        os.makedirs(vaspstdinputdir)
-
     vaspstdkpointsdict = {
         "comment": "Kpoints from doped.vasp_std_files",
         "generation_style": "Gamma",  # Set to Monkhorst for Monkhorst-Pack generation
@@ -436,15 +414,17 @@ def vasp_std_files(
 
 
 def vasp_ncl_files(
-    single_defect_dict: dict, input_dir: str = None, incar_settings: dict = None
+        single_defect_dict: dict,
+        input_dir: str = None,
+        incar_settings: dict = None,
+        kpoints_settings: dict = None,
+        potcar_settings: dict = None,
 ) -> None:
     """
-    Generates INCAR for vasp_ncl single-shot SOC energy calculation on
-    vasp_std-relaxed defect structure.
-    For POSCAR, POTCAR, KPOINTS, use on command-line (to continue on from vasp_std run):
-    'cp vasp_std/CONTCAR vasp_ncl/POSCAR; cp vasp_std/{POTCAR,CHGCAR} vasp_ncl/'
-    and 'cp vasp_std/IBZKPT vasp_ncl/KPOINTS' because you need to use non-symmetrised k-points
-    (single-weighted) for accurate SOC calculations.
+    Generates INCAR, POTCAR and non-symmetrised KPOINTS for vasp_ncl single-shot SOC energy
+    calculation on vasp_std-relaxed defect structure.
+    For POSCAR, use on command-line (to continue on from vasp_std run):
+    'cp vasp_std/CONTCAR vasp_ncl/POSCAR'
     Args:
         single_defect_dict (dict):
             Single defect-dictionary from prepare_vasp_defect_inputs()
@@ -459,8 +439,54 @@ def vasp_ncl_files(
             Highly recommended to look at output INCARs or doped.vasp_input
             source code, to see what the default INCAR settings are.
             (default: None)
+        kpoints_settings (dict):
+            Dictionary of user KPOINTS settings (in pymatgen Kpoints.from_dict() format). Most
+            common option would be {"kpoints": [[3, 3, 1]]} etc.
+            Will generate a non-symmetrised (i.e. explicitly listed) KPOINTS, as typically required
+            for vasp_ncl calculations. Default is Gamma-centred 2 x 2 x 2 mesh.
+            (default: None)
+        potcar_settings (dict):
+            Dictionary of user POTCAR settings to override default settings.
+            Highly recommended to look at `default_potcar_dict` from doped.vasp_input to see what
+            the (Pymatgen) syntax and doped default settings are.
+            (default: None)
     """
-    nelect = single_defect_dict["NELECT"]
+    supercell = single_defect_dict["Defect Structure"]
+
+    # Directory
+    vaspnclinputdir = input_dir + "/vasp_ncl/" if input_dir else "VASP_Files/vasp_ncl/"
+    if not os.path.exists(vaspnclinputdir):
+        os.makedirs(vaspnclinputdir)
+
+    warnings.filterwarnings(
+        "ignore", category=BadInputSetWarning
+    )  # Ignore POTCAR warnings because Pymatgen incorrectly detecting POTCAR types
+
+    potcar_dict = deepcopy(default_potcar_dict)
+    if potcar_settings:
+        potcar_dict["POTCAR"].update(potcar_settings.pop("POTCAR"))
+        potcar_dict.update(potcar_settings)
+    defect_relax_set = DefectRelaxSet(supercell, charge=single_defect_dict["Transformation "
+                                                                           "Dict"]["charge"],
+                                      user_potcar_settings=potcar_dict["POTCAR"],
+                                      user_potcar_functional=potcar_dict["POTCAR_FUNCTIONAL"])
+    potcars = _check_psp_dir()
+    if not potcars:
+        warnings.warn("POTCAR directory not set up with pymatgen, so no input files will be "
+                      "generated (you should use vasp_input.vasp_gam_files to create the initial "
+                      "relaxation files, then continue from this pre-converged structure with "
+                      "vasp_std and finally vasp_ncl if SOC important)")
+        return  # exit here
+    defect_relax_set.potcar.write_file(vaspnclinputdir + "POTCAR")
+
+    relax_set_incar = defect_relax_set.incar
+    try:
+        # Only set if change in NELECT
+        nelect = relax_set_incar.as_dict()["NELECT"]
+    except KeyError:
+        # Get NELECT if no change (KV = -dNELECT = 0)
+        nelect = defect_relax_set.nelect
+
     # Variable parameters first
     vaspnclincardict = {
         "# May need to change NELECT, NCORE, KPAR, AEXX, ENCUT, NUPDOWN": "variable parameters",
@@ -474,7 +500,7 @@ def vasp_ncl_files(
         "ICORELEVEL": "0 # Get core potentials in OUTCAR for Kumagai corrections",
         "NSW": 0,
         "LSORBIT": True,
-        "EDIFF": 1e-06,
+        "EDIFF": 1e-06, # tight for final energy and converged DOS
         "EDIFFG": -0.01,
         "ALGO": "All",
         "ADDGRID": True,
@@ -497,12 +523,31 @@ def vasp_ncl_files(
         "SIGMA": 0.05,
     }
     if incar_settings:
+        for k in incar_settings.keys():  # check INCAR flags and warn if they don't exist (typos)
+            if k not in incar_params.keys():  # this code is taken from pymatgen.io.vasp.inputs
+                warnings.warn(  # but only checking keys, not values so we can add comments etc
+                    "Cannot find %s from your incar_settings in the list of INCAR flags" % (k),
+                    BadIncarWarning,
+                )
         vaspnclincardict.update(incar_settings)
 
-    # Directory
-    vaspnclinputdir = input_dir + "/vasp_ncl/" if input_dir else "VASP_Files/vasp_ncl/"
-    if not os.path.exists(vaspnclinputdir):
-        os.makedirs(vaspnclinputdir)
+    k_grid = kpoints_settings.pop("kpoints")[0] if (kpoints_settings and
+                                                "kpoints" in kpoints_settings) else [2,2,2]
+    shift = np.array(kpoints_settings.pop("usershift")) if (kpoints_settings and
+                                                "usershift" in kpoints_settings) else np.array([
+        0,0,0])
+    vasp_ncl_kpt_array = monkhorst_pack(k_grid)
+    vasp_ncl_kpt_array += -vasp_ncl_kpt_array[0] + shift # Gamma-centred by default
+    vasp_ncl_kpts = Kpoints(comment="Non-symmetrised KPOINTS for vasp_ncl, from doped",
+            num_kpts=len(vasp_ncl_kpt_array),
+            style=Kpoints_supported_modes(5), # Reciprocal
+            kpts=vasp_ncl_kpt_array,
+            kpts_weights=[1, ] * len(vasp_ncl_kpt_array))
+    if kpoints_settings:
+        modified_kpts_dict = vasp_ncl_kpts.as_dict()
+        modified_kpts_dict.update(kpoints_settings)
+        vasp_ncl_kpts = Kpoints.from_dict(modified_kpts_dict)
+    vasp_ncl_kpts.write_file(vaspnclinputdir + "KPOINTS")
 
     vaspnclincar = Incar.from_dict(vaspnclincardict)
 
@@ -532,6 +577,7 @@ def vasp_converge_files(
     structure: "pymatgen.core.Structure",
     input_dir: str = None,
     incar_settings: dict = None,
+    potcar_settings: dict = None,
     config: str = None,
 ) -> None:
     """
@@ -553,6 +599,11 @@ def vasp_converge_files(
             CONFIG file string. If provided, will also write the CONFIG file (to automate
             convergence tests with vaspup2.0) to each 'input' directory.
             (default: None)
+        potcar_settings (dict):
+            Dictionary of user POTCAR settings to override default settings.
+            Highly recommended to look at `default_potcar_dict` from doped.vasp_input to see what
+            the (Pymatgen) syntax and doped default settings are.
+            (default: None)
     """
 
     # Variable parameters first
@@ -569,7 +620,7 @@ def vasp_converge_files(
         "GGA": "PS",
         "ALGO": "Normal",
         "ADDGRID": True,
-        "EDIFF": 1e-07,
+        "EDIFF": 1e-06,
         "EDIFFG": -0.01,
         "IBRION": -1,
         "ICHARG": 1,
@@ -587,6 +638,13 @@ def vasp_converge_files(
     if all(is_metal(element) for element in structure.composition.elements):
         vaspconvergeincardict["ISMEAR"] = "2 # Metal, use Methfessel-Paxton smearing scheme"
     if incar_settings:
+        for k in incar_settings.keys():  # check INCAR flags and warn if they don't exist (
+            # typos)
+            if k not in incar_params.keys():  # this code is taken from pymatgen.io.vasp.inputs
+                warnings.warn(  # but only checking keys, not values so we can add comments etc
+                    "Cannot find %s from your incar_settings in the list of INCAR flags" % (k),
+                    BadIncarWarning,
+                )
         vaspconvergeincardict.update(incar_settings)
 
     # Directory
@@ -594,111 +652,21 @@ def vasp_converge_files(
     if not os.path.exists(vaspconvergeinputdir):
         os.makedirs(vaspconvergeinputdir)
 
-    vasppotcardict = {
-        "POTCAR_FUNCTIONAL": "PBE",  # May need to change this if you've got
-        # different POTCARs
-        "POTCAR": {
-            "Ac": "Ac",
-            "Ag": "Ag",
-            "Al": "Al",
-            "Ar": "Ar",
-            "As": "As",
-            "Au": "Au",
-            "B": "B",
-            "Ba": "Ba_sv",
-            "Be": "Be_sv",
-            "Bi": "Bi",
-            "Br": "Br",
-            "C": "C",
-            "Ca": "Ca_sv",
-            "Cd": "Cd",
-            "Ce": "Ce",
-            "Cl": "Cl",
-            "Co": "Co",
-            "Cr": "Cr_pv",
-            "Cs": "Cs_sv",
-            "Cu": "Cu_pv",
-            "Dy": "Dy_3",
-            "Er": "Er_3",
-            "Eu": "Eu",
-            "F": "F",
-            "Fe": "Fe_pv",
-            "Ga": "Ga_d",
-            "Gd": "Gd",
-            "Ge": "Ge_d",
-            "H": "H",
-            "He": "He",
-            "Hf": "Hf_pv",
-            "Hg": "Hg",
-            "Ho": "Ho_3",
-            "I": "I",
-            "In": "In_d",
-            "Ir": "Ir",
-            "K": "K_sv",
-            "Kr": "Kr",
-            "La": "La",
-            "Li": "Li_sv",
-            "Lu": "Lu_3",
-            "Mg": "Mg_pv",
-            "Mn": "Mn_pv",
-            "Mo": "Mo_pv",
-            "N": "N",
-            "Na": "Na_pv",
-            "Nb": "Nb_pv",
-            "Nd": "Nd_3",
-            "Ne": "Ne",
-            "Ni": "Ni_pv",
-            "Np": "Np",
-            "O": "O",
-            "Os": "Os_pv",
-            "P": "P",
-            "Pa": "Pa",
-            "Pb": "Pb_d",
-            "Pd": "Pd",
-            "Pm": "Pm_3",
-            "Pr": "Pr_3",
-            "Pt": "Pt",
-            "Pu": "Pu",
-            "Rb": "Rb_sv",
-            "Re": "Re_pv",
-            "Rh": "Rh_pv",
-            "Ru": "Ru_pv",
-            "S": "S",
-            "Sb": "Sb",
-            "Sc": "Sc_sv",
-            "Se": "Se",
-            "Si": "Si",
-            "Sm": "Sm_3",
-            "Sn": "Sn_d",
-            "Sr": "Sr_sv",
-            "Ta": "Ta_pv",
-            "Tb": "Tb_3",
-            "Tc": "Tc_pv",
-            "Te": "Te",
-            "Th": "Th",
-            "Ti": "Ti_pv",
-            "Tl": "Tl_d",
-            "Tm": "Tm_3",
-            "U": "U",
-            "V": "V_pv",
-            "W": "W_pv",
-            "Xe": "Xe",
-            "Y": "Y_sv",
-            "Yb": "Yb_2",
-            "Zn": "Zn",
-            "Zr": "Zr_sv",
-        },
-    }
+    warnings.filterwarnings(
+        "ignore", category=BadInputSetWarning
+    )  # Ignore POTCAR warnings because Pymatgen incorrectly detecting POTCAR types
+    potcar_dict = default_potcar_dict
+    if potcar_settings:
+        potcar_dict["POTCAR"].update(potcar_settings.pop("POTCAR"))
+        potcar_dict.update(potcar_settings)
+    vaspconvergeinput = DictSet(structure, config_dict=potcar_dict)
+    vaspconvergeinput.potcar.write_file(vaspconvergeinputdir + "POTCAR")
+
     vaspconvergekpts = Kpoints().from_dict(
         {"comment": "Kpoints from vasp_gam_files", "generation_style": "Gamma"}
     )
     vaspconvergeincar = Incar.from_dict(vaspconvergeincardict)
-    warnings.filterwarnings(
-        "ignore", category=BadInputSetWarning
-    )  # Ignore POTCAR warnings because Pymatgen incorrectly detecting POTCAR types
-    vaspconvergeinput = DictSet(structure, config_dict=vasppotcardict)
 
-    vaspconvergeinput.potcar.write_file(vaspconvergeinputdir + "POTCAR")
     vaspconvergeposcar = Poscar(structure)
     vaspconvergeposcar.write_file(vaspconvergeinputdir + "POSCAR")
     with zopen(vaspconvergeinputdir + "INCAR", "wt") as incar_file:
